/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.controller;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import java.io.IOException;

import org.oransc.rappmanager.exception.ServiceException;
import org.oransc.rappmanager.service.App;
import org.oransc.rappmanager.service.AppList;
import org.oransc.rappmanager.service.AppService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController("rAppController")
@RequestMapping("rms")
@Api(tags = {"rapp"})
public class AppController {

    @Autowired
    private AppService appService;

    @Autowired
    private static Gson gson = new GsonBuilder().create();

    @GetMapping(path = "/apps", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Return all Apps")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "rApp List")})
    public ResponseEntity<AppList> getAllApps() throws ServiceException {
        return new ResponseEntity<>(appService.getAllApps(), HttpStatus.OK);
    }

    @PostMapping(
        path = "/install",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Install the rApp")
    @ApiResponses(value = {@ApiResponse(code = 201, message = "rApp Installed")})
    public ResponseEntity<Object> installApp(@RequestBody InstallationInfo info) throws ServiceException {
        App app = appService.getApp(info.getName(), info.getVersion());
        appService.installApp(app);
        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    @DeleteMapping(path = "/install/{name}/{version}", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Uninstall the App")
    @ApiResponses(value = {@ApiResponse(code = 201, message = "rApp Uninstalled")})
    public ResponseEntity<Object> deleteApp( //
        @PathVariable("name") String name, //
        @PathVariable("version") String version) throws ServiceException {
        App app = appService.getApp(name, version);
        appService.uninstallApp(app);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @PostMapping(
        path = "/apps",
        consumes = MediaType.MULTIPART_FORM_DATA_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Onboard the Chart")
    @ApiResponses(value = {@ApiResponse(code = 201, message = "Chart Onboarded")})
    public ResponseEntity<String> onboardChart( //
        @RequestPart("chart") MultipartFile chartFile, //
        @RequestPart("values") MultipartFile overrideFile, //
        @RequestParam("info") String infoJson) throws ServiceException, IOException {

        AppInfo info = gson.fromJson(infoJson, AppInfo.class);
        appService.saveApp(info, chartFile, overrideFile);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @DeleteMapping(path = "/apps/{name}/{version}")
    @ApiOperation(value = "Delete the app")
    @ApiResponses(value = {@ApiResponse(code = 204, message = "Chart Deleted")})
    public ResponseEntity<Object> deleteChart(@PathVariable("name") String name,
        @PathVariable("version") String version) throws ServiceException {
        App app = appService.getApp(name, version);
        appService.deleteApp(app);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

}
